#!/usr/bin/env python
#
# Copyright (C) 2008 Real Time Enterprises, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os,sys
import getopt
from realtime.template import GenshiTemplate
from realtime.observable import Subject, Listener
from genshi.template import TextTemplate, TemplateNotFound
from raidtracker.raidtracker import RaidTracker
from raidtracker.parser import Parser

__app__ = 'RaidTracker for GLA'
__copyright__ = 'Copyright (C) 2009 Real Time Enterprises, Inc.'
__version__ = '0.1'
__min_version__ = (2, 4)

def version():
    print '%s %s' % (__app__, __version__)
    print 'Using python interpreter:', sys.executable
    import site
    print 'Using python standard library:', os.path.dirname(site.__file__)
    print
    print __copyright__
    print
    print '"%s" comes with ABSOLUTELY NO WARRANTY. "%s"' % (__app__, __app__)
    print 'is free software, and you may use, modify and redistribute it under'
    print 'the terms of the GNU General Public License version 2.'
    
def usage():
    print >> sys.stderr, """
    Usage: RaidTracker [-h] [-v] -f RaidTracker.lua
    
    Options:
    -h, --help
        Print detailed help screen
    -v, --version
        Print version information
    -f, --file
        RaidTracker.lua file
        
    Send email to tanner@real-time.com if you have questions regarding
    this software.
    """
def missing_required(option):
    print >> sys.stderr, """Error: Missing %s 
    The '%s' is a required option.""" % (option, option)
                                     
def main():
    
    # Name of RaidTracker.lua file
    lua_file = None
    
    if sys.version_info < __min_version__:
        print >> sys.stderr, '%s: error: cannot find a suitable python interpreter' % __app__
        print >> sys.stderr, '    Need python %d.%d or later' % __min_version__
        print >> sys.stderr, '    Found python %d.%d' % (sys.version_info[0], sys.version_info[1])
        sys.exit(1)
        
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'hvf:', ['help', 'version', 'file='])
    except getopt.GetoptError:
        usage()
        sys.exit(0)

    for o, a in opts:
        if o in ('-v', '--version'):
            version()
            sys.exit(0)
        if o in ('-h', '--help'):
            usage()
            sys.exit(0)
        if o in ('-f', '--file'):
            lua_file = a
    
    if lua_file == None:
        missing_required('--file')
        sys.exit(1)

    # Setup observer pattern
    subject = Subject()
    
    try:
        bbcode_template = GenshiTemplate('../raidtrackertg2/templates', 'gla_forum.bbcode', TextTemplate)     
        raid_tracker = RaidTracker(bbcode_template)
        
        # Register raid_tracker with observer
        subject.register(raid_tracker)
        
        parser = Parser()
        parser.open(lua_file)
        parser.parseLUA('KARaidTrackerDB')
    except RuntimeError, (e):
        print >> sys.stderr, 'Runtime Error:', e
        sys.exit(1)
    except TemplateNotFound, (e):
        print >> sys.stderr, 'Template Error:', e
        sys.exit(1)
    except:
        print >> sys.stderr, 'RaidTracker open or parse failed! Terrible error message, clean this up!', sys.exc_info()[0]
        sys.exit(1)
 
    loots = parser.getLoots('Epic')
    subject.notify_listeners(loots)
    
    #template.generate(loots=loots)
    #print template.render('text')

if __name__ == '__main__':
    main()
