"""Main Controller"""
from raidtrackertg2.lib.base import BaseController
from tg import expose, flash, require
import tg
from tg import redirect
from pylons.i18n import ugettext as _
from tg import redirect, validate
from raidtrackertg2.model import DBSession, metadata
from catwalk.tg2 import Catwalk
from repoze.what import predicates
from raidtrackertg2.controllers.secc import Secc
from tw.forms import TableForm, FileField, SingleSelectField, RadioButtonList
from tw.api import WidgetsList
from sqlalchemy.orm import join, aliased
from sqlalchemy import desc


from raidtracker.raidtracker import RaidTracker
from raidtracker.parser import Parser
from raidtrackertg2.model.raid import Raid
from raidtrackertg2.model.loot import Loot
from raidtrackertg2.model.item import Item
from raidtrackertg2.model.player import Player
from raidtrackertg2.model.guildrank import GuildRank
from raidtrackertg2.model.professions import Professions
import tempfile
from datetime import datetime

class RaidtrackerError(Exception):
    def __init__(self, value):
        self.value = value
        
    def __str__(self):
        return repr(self.value)
        
class UploadForm(TableForm):
    class fields(WidgetsList):
        datafile = FileField(label='RaidTracker:')
#        viewchoices = (('bbcode', 'Inline bbcode'),
#                        ('popup_bbcode', 'Popup bbcode'),
#                        ('zipfile', 'Download as zipfile'),
#                        ('wowhead', 'WowHead'))
#        view = SingleSelectField(options=viewchoices, default='inline_bbcode')
        
        filterchoices = (
            ('Heirloom', 'Heirloom (light gold)'),
            ('Artifact', 'Artifact (light gold)'),
            ('Lengendary', 'Legendary (orange)'),
            ('Epic', 'Epic (purple)'),
            ('Rare', 'Rare (blue)'),
            ('Uncommon', 'Uncommon (green)'),
            ('Common', 'Common (white)'),
            ('Poor', 'Poor (grey)'))
        filter = SingleSelectField(options=filterchoices, default='Epic')
        
        viewchoices = ['bbcode', 'html']
        view = RadioButtonList(options=viewchoices, 
            default='bbcode')

upload_form = UploadForm("upload_form",
    submit_text = 'Upload',
    action = '/parselua',
)
            
class RaidLuaForm(TableForm):
    class fields(WidgetsList):
        luafile = FileField(label='RaidTracker.lua:')        

raidlua_form = RaidLuaForm('raidlua_form',
    submit_text = 'Upload',
    action = '/parselua'
)    

class SecuredCatwalk(Catwalk):
    allow_only = predicates.has_permission('manage',
                            msg=_('Only for people with the "manage" permission'))
        
class RootController(BaseController):
    
    admin = SecuredCatwalk(DBSession, metadata)

    secc = Secc()

    def getLoots(self, datafile, filter):
        lua = datafile.file.read()
                
        temp = tempfile.NamedTemporaryFile()
        temp.write(lua)
        temp.flush()
        
        raid_tracker = RaidTracker()
        raid_tracker.open(temp.name)
        raid_tracker.parseLUA('KARaidTrackerDB')
        loots = raid_tracker.getLoots(filter)
        temp.close()
        return loots
    
    def post_mkstemp(self, post_data):
        raidtracker_lua = post_data.file.read()
        temp = tempfile.NamedTemporaryFile()
        temp.write(raidtracker_lua)
        temp.flush()
        return temp
    
    def version_check(self, lua, version_copy, version_fix):
        vc = lua['Options']['VersionCopy']
        if version_copy != vc:
            raise RaidtrackerError, 'Unsupported VersionCopy'
        
        vf = lua['Options']['VersionFix']
        if version_fix != vf:
            raise RaidtrackerError, 'Unsupported VersionFix'
            
    @expose()
    def parselua(self, luafile):
        parser = Parser()
        
        try:
            raidtracker_lua = self.post_mkstemp(luafile)
            parser.open(raidtracker_lua.name)
            lua = parser.eval('KARaidTrackerDB')
            
            # All sort of checks here, maybe observer pattern?
            self.version_check(lua, 1, 27)
            
            logs = lua['Log']
            for index in logs:
                
                raid = Raid.parse(index, logs)
                try:
                    DBSession.query(Raid).filter(Raid.raid_id == raid.raid_id).one()
                except Exception, (e):
                    if raid.raid_id != None:
                        DBSession.add(raid)
                    
                players = Player.parse(index, logs)
                for player in players:
                    try:
                        DBSession.query(Player).filter(Player.name == player.name).one()
                    except Exception, (e):
                        # No player name was found so add player to the table
                       DBSession.add(player)
                
                items = Item.parse(index, logs)
                for item in items:
                    try:
                        DBSession.query(Item).filter(Item.item_id == item.item_id).one()
                    except Exception, (e):
                        DBSession.add(item)
                        
                loots = Loot.parse(index, logs, raid.raid_id)
                for loot in loots:
                    player_id = DBSession.query(Player.id).filter(Player.name==loot.player_id).one()
                    loot.player_id = player_id[0]
                    DBSession.add(loot)
                    
        except RaidtrackerError, (e):
            flash('Error: '+ e.value)
            raise tg.redirect('error')
        except RuntimeError, (e):
            flash(e)
            raise tg.redirect('error')
        finally:
            raidtracker_lua.close() 
                   
        redirect('/raids')
        
    @expose('raidtrackertg2.templates.raids')
    def raids(self):
        raids = DBSession.query(Raid).order_by(Raid.raid_id.desc())                        
        return dict(raids=raids, page='raids')
        
    @expose('raidtrackertg2.templates.upload')
    def upload(self):
        return dict(form=raidlua_form, page='upload')
    
    @expose('raidtrackertg2.templates.players')
    def players(self):
        players = DBSession.query(Player, GuildRank.name).\
            filter(Player.rank_number==GuildRank.rank_number).\
            filter(Player.id==Professions.player_id).\
            order_by(Player.name)

        return dict(players=players, page='players')
    
    @expose('raidtrackertg2.templates.items')
    def items(self):
        items = DBSession.query(Item).order_by(Item.name)
        return dict(items=items, page='items')
            
    @expose('raidtrackertg2.templates.loots')
    def loots(self, raid_id):
        loots = DBSession.query(Loot, Item, Player).\
            filter(Loot.raid_id==raid_id).\
            filter(Loot.item_id==Item.item_id).\
            filter(Loot.player_id==Player.id)
        return dict(loots=loots, page='loots')
    
    @expose('raidtrackertg2.templates.players')
    def profession(self, profession):
        players = DBSession.query(Player, GuildRank.name).\
            filter(Player.rank_number==GuildRank.rank_number).\
            filter(Player.id==Professions.player_id).\
            filter(Professions.name.like(profession+'%')).\
            order_by(desc(Professions.player_skill))  
        return dict(players=players, page='professions')
        
    @expose('raidtrackertg2.templates.view-old')
    def viewold(self, datafile, filter, view):
        loots = self.getLoots(datafile, filter)
        return dict(loots=loots, page=view, view=view)
        
    @expose('raidtrackertg2.templates.index')
    def index(self):
        return dict(page='welcome')

    @expose('raidtrackertg2.templates.about')
    def about(self):
        return dict(page='about')
        
    @expose('raidtrackertg2.templates.error')
    def error(self):
            return dict(page='error')

#    @expose('raidtrackertg2.templates.authentication')
#    def auth(self):
#        return dict(page='auth')

    @expose('raidtrackertg2.templates.index')
    @require(predicates.has_permission('manage', msg=_('Only for managers')))
    def manage_permission_only(self, **kw):
        return dict(page='managers stuff')

    @expose('raidtrackertg2.templates.index')
    @require(predicates.is_user('editor', msg=_('Only for the editor')))
    def editor_user_only(self, **kw):
        return dict(page='editor stuff')

    @expose('raidtrackertg2.templates.login')
    def login(self, **kw):
        came_from = kw.get('came_from', '/')
        return dict(page='login', header=lambda *arg: None,
                    footer=lambda *arg: None, came_from=came_from)
