# Copyright (C) 2009 Real Time Enterprises, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from sqlalchemy import *
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Integer, Unicode
#from sqlalchemy.orm import relation, backref
from raidtrackertg2.model import DeclarativeBase, metadata, DBSession

# http://www.wowwiki.com/API_GetItemInfo
# http://www.wowwiki.com/ItemString
class Item(DeclarativeBase):
    __tablename__ = 'items'
    
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(255), nullable=False)
    quality = Column(Unicode(255), nullable=False)
    item_id = Column(Integer, unique=True)
    enchant_id = Column(Integer)
    jewel1 = Column(Integer)
    jewel2 = Column(Integer)
    jewel3 = Column(Integer)
    jewel4 = Column(Integer)
    suffix_id = Column(Integer)
    unique_id = Column(Integer)
 
    def __init__(self, name, quality, item_id, enchant_id, jewel1, jewel2, jewel3, jewel4, suffix_id, unique_id):
        self.name = name
        self.quality = quality
        self.item_id = item_id
        self.enchant_id = enchant_id
        self.jewel1 = jewel1
        self.jewel2 = jewel2
        self.jewel3 = jewel3
        self.jewel4 = jewel4
        self.suffix_id = suffix_id
        self.unique_id = unique_id

    @staticmethod
    def parse(index, logs):
        loots = logs[index]['Loot']
        for index in loots:
            loot_id = loots[index]

            item = Item.parse2(loot_id['item'])

            yield item
                    
    @staticmethod    
    def parse2(item):                
        name = item['name']
        
        quality_hex = item['c']
        #self.quality = self.__quality__[quality_hex]
        
        item_string = item['id']
        split = item_string.split(':')
        item_id = int(split[0])
        
        enchant_id = int(split[1])
        jewel1_id = split[2]
        suffix_id = split[6]
        unique_id = split[7]
        return Item(name, quality_hex, item_id, enchant_id, jewel1_id, 2, 3, 4, suffix_id, unique_id)
        
    def getName(self):
        return self.name
       
    def getQuality(self):
        return self.quality
        
    def getItemID(self):
        return self.itemID

# http://www.wowwiki.com/API_ITEM_QUALITY_COLORS
# http://www.wowwiki.com/API_GetItemQualityColor
    __quality__ = {
        'ff9d9d9d' : 'Poor',
        'ffffffff' : 'Common',
        'ff1eff00' : 'Uncommon',
        'ff0070dd' : 'Rare', 
        'ffa335ee' : 'Epic', 
        'ffff8000' : 'Legendary', 
        'ffe6cc80' : 'Artifact',
        'ffe6cc80' : 'Heirloom'
    }

