# Copyright (C) 2009 Real Time Enterprises, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

from sqlalchemy import *
from sqlalchemy.orm import mapper, relation
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Integer, Unicode
from sqlalchemy.orm import relation, backref
from raidtrackertg2.model import DeclarativeBase, metadata, DBSession
from datetime import datetime

def __format_class__(the_class):
        if the_class != None:
            return the_class.lower()
        return the_class

def __format_sex__(sex):
        lookup = ['unknown', 'unknown', 'male', 'female']
        return lookup[sex]
        
class Player(DeclarativeBase):
    __tablename__ = 'players'
    
    id = Column(Integer, primary_key=True)
    name = Column(Unicode(255), nullable=False, unique=True)
    race = Column(Unicode(255), nullable=False)
    guild = Column(Unicode(255), nullable=True)
    sex = Column(Unicode(255), nullable=False)
    the_class = Column(Unicode(255), nullable=False)
    level = Column(Integer)
    rank_number = Column(Integer, ForeignKey('guild_rank.rank_number'))
    achievement_points = Column(Integer)
    professions = relation('Professions', backref='player')
    
    def __init__(self, name, race, guild, sex, the_class, level, 
                rank_number=-1, achievement_points=0):
        self.name = name
        self.race = race
        self.guild = guild
        self.sex = sex
        self.the_class = the_class
        self.level = level
        self.rank_number = rank_number
        self.achievement_points = achievement_points

    @staticmethod
    def parse(index, logs):
        players = logs[index]['Players']
        for index in players:
            
            # Handle the weird 'Unknown' player names from Cadin's .lua
            if index == 'Unknown':
                continue
                
            player = players[index]
            sex = __format_sex__(player['sex'])
            the_class = __format_class__(player['class'])
            yield Player(index, player['race'], player['guild'], sex, the_class, player['level'])
