#
# Copyright (C) 2008 Real Time Enterprises, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class Listener:
    def __init__(self, name, subject):
        self.name = name
        subject.register(self)
 
    def notify(self, event):
        print self.name, "received event", event
 
class Subject:
    """ 
    An implementation of the Observer pattern in Python (with event arguments). 
    """
    def __init__(self):
        """ 
        Initialize a `Subject` object.
        """
        self.listeners = []

    def register(self, listener):
        """ 
        Add an object to receive notifications. 
        """
        self.listeners.append(listener)
 
    def unregister(self, listener):
        """ 
        Remove an object from the list of observers. 
        """ 
        self.listeners.remove(listener)
 
    def notify_listeners(self, event):
        """
        Notify objects of event. 
        """
        for listener in self.listeners:
            listener.notify(event)