# Copyright (C) 2008 Real Time Enterprises, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os.path
from realtime.misc import deprecated
from genshi.template import TemplateLoader
from genshi.template import MarkupTemplate
from genshi.template import TextTemplate

class GenshiTemplate:
	"""
	Base class for generating genshi templates formatted in text or (default) html.
	"""
	def __init__(self, search, template, cls=MarkupTemplate):
		""" 
		Create a `GenshiTemplate` object.

		Parameters:

		- `search` - directories to search for the genshi templates
		- `template` - genshi template to use
		- `cls` - the class of the template object to instantiate
		"""
		print "__file__:%s" % __file__
		loader = TemplateLoader(os.path.join(os.path.dirname(__file__), search), auto_reload=True)
		self.tmpl = loader.load(template, None, cls, None)
        
	def generate(self, **kwargs):
		"""
		Replace all vaiables in the genshi template with values.
		
		Parameter:

		- `kwargs` - keyword arguements used to fill in the genshi template variables.
		"""
		self.stream = self.tmpl.generate(**kwargs)
        
	def render(self, format):
		"""
		Return the genshi template as a formatted stream.

		Parameter:

		- `format` - format to render the stream (text or html)
		"""
		return self.stream.render(format)

