from sqlalchemy import create_engine
from sqlalchemy import Table, Column, Integer, String, MetaData, DateTime, ForeignKey
from sqlalchemy.orm import mapper, sessionmaker, relation, backref
from sqlalchemy.ext.declarative import declarative_base
from datetime import datetime
import time
import lua
from raidtracker.parser import Parser

Base = declarative_base()

class Item(Base):
    __tablename__ = 'items'
    __quality__ = {
        'ff9d9d9d' : 'Poor',
        'ffffffff' : 'Common',
        'ff1eff00' : 'Uncommon',
        'ff0070dd' : 'Rare', 
        'ffa335ee' : 'Epic', 
        'ffff8000' : 'Legendary', 
        'ffe6cc80' : 'Artifact',
        'ffe6cc80' : 'Heirloom'
    }
    
    id = Column(Integer, primary_key=True)
    name = Column(String, nullable=False)
    quality = Column(String, nullable=False)
    itemID = Column(Integer, unique=True)
    enchantID = Column(Integer)
    j1 = Column(Integer)
    j2 = Column(Integer)
    j3 = Column(Integer)
    j4 = Column(Integer)
    suffixID = Column(Integer)
    uniqueID = Column(Integer)
    
    def __init__(self, name, quality, itemID, enchantID, j1, j2, j3, j4, suffixID, uniqueID):
        self.name = name
        self.quality = quality
        self.itemID = itemID
        self.enchantID = enchantID
        self.j1 = j1
        self.j2 = j2
        self.j3 = j3
        self.j4 = j4
        self.suffixID = suffixID
        self.uniqueID = uniqueID
        
class Loot(Base):
    __tablename__ = 'lewtz'
    
    id = Column(Integer, primary_key=True)
    toon = Column(String, nullable=False)
    item_id = Column(Integer, ForeignKey('items.itemID'))
    time = Column(DateTime)
    note = Column(String, nullable=False)
    
    item = relation(Item, backref=backref('lewtz', order_by=id))
    
    def setToon(self, toon):
        self.toon = toon
    
    def setItem(self,item):
        self.item = item
    
    def setTime(self, time):
        self.time = time
    
    def setNote(self, note):
        self.note = note

engine = create_engine('sqlite:///spike.db', echo=False)
Session = sessionmaker(bind=engine)
Base.metadata.create_all(engine) 
session = Session()

parser = Parser()
parser.open('RaidTracker.lua')
parser.parseLUA('KARaidTrackerDB')
loots = parser.getLoots('Epic')
for loot in loots:
    try:
        session.query(Item).filter(Item.itemID == loot.item.itemID).one()
        print 'Duplicate:', loot.item.name
    except Exception, e:
        # No row was found so add item to the table
        item = Item(loot.item.name, loot.item.quality, loot.item.itemID, loot.item.enchantID, loot.item.j1, loot.item.j2, loot.item.j3, 
                loot.item.j4, loot.item.suffixID, loot.item.uniqueID)
        session.add(item)
    
# toon, item, time, note
#loot = Loot()
#loot.setToon('Linghu')
#loot.setItem(item)
#loot.setTime(datetime.now())
#loot.setNote('My Note')

#session.add(loot)
session.commit()
